
#include <windows.h>
#include <stdio.h>

void PrintCert(PCCERT_CONTEXT pCert);

char *providers[] = {
    "<UNKNOWN> (0)",
    "PROV_RSA_FULL (1)",
    "PROV_RSA_SIG (2)",
    "PROV_DSS (3)",
    "PROV_FORTEZZA (4)",
    "PROV_MS_EXCHANGE (5)",
    "PROV_SSL (6)",
    "<UNKNOWN> (7)",
    "<UNKNOWN> (8)",
    "<UNKNOWN> (9)",
    "<UNKNOWN> (10)",
    "<UNKNOWN> (11)",
    "PROV_RSA_SCHANNEL (12)",
    "PROV_DSS_DH (13)",
    "PROV_EC_ECDSA_SIG (14)",
    "PROV_EC_ECNRA_SIG (15)",
    "PROV_EC_ECDSA_FULL (16)",
    "PROV_EC_ECNRA_FULL (17)",
    "PROV_DH_SCHANNEL (18)",
    "<UNKNOWN> (19)",
    "PROV_SPYRUS_LYNKS (20)",
    "PROV_RNG (21)",
    "PROV_INTEL_SEC (22)",
    "PROV_REPLACE_OWF (23)",
    "PROV_RSA_AES (24)"
};

int main(int argc, char* argv[])
{
HCERTSTORE      hStore        = NULL;
PCCERT_CONTEXT  pCert         = NULL;
LPWSTR          pSubjectName  = NULL;
DWORD           length        = 0;
int             status        = ERROR_SUCCESS;

	__try
	{
        printf("checkcert - Steve Johnson (www.stevestechspot.com)\r\n\r\n");

        if(argc != 3)
        {
            printf("Usage: \r\n");
            printf("checkcert STORENAME SUBJECTNAME\r\n");
            printf(" STORENAME - Name of a LocalMachine certificate store (MY, TRUST, etc...)\r\n");
            printf(" SUBJECTNAME - String to match in the certificate subject name\r\n\r\n");
            return ERROR_INVALID_PARAMETER;
        }

		length = MultiByteToWideChar(CP_ACP, 0, argv[2], -1, NULL, 0);

		pSubjectName = new WCHAR[length];

		MultiByteToWideChar(CP_ACP, 0, argv[2], -1, pSubjectName, length);

		HCERTSTORE hStore = CertOpenStore(
            CERT_STORE_PROV_SYSTEM_A,
            X509_ASN_ENCODING,
            0,
            CERT_SYSTEM_STORE_LOCAL_MACHINE | CERT_STORE_OPEN_EXISTING_FLAG | CERT_STORE_READONLY_FLAG,
            argv[1]);

		if(NULL == hStore)
		{
            status = GetLastError();

            if(ERROR_FILE_NOT_FOUND == status)
			{
				printf("The store name is invalid.\r\n");
			}
			else
			{
				printf("Cannot open the store.  CertOpenStore failed.  Error = 0x%08X", status);
			}
			
			__leave;
		}

        BOOL noneFound = TRUE;

        do
        {
            if(FALSE == noneFound)
            {
                printf("\r\n\r\n");
            }

		    pCert = CertFindCertificateInStore(hStore, X509_ASN_ENCODING, 0, CERT_FIND_SUBJECT_STR, pSubjectName, pCert);
		    if(NULL == pCert)
		    {
                if(FALSE == noneFound)
                {
                    break;
                }

			    status = GetLastError();
                printf("Cannot access the certificate.  CertFindCertificateInStore failed.  Error = 0x%08X", status);			
			    __leave;
		    }

            noneFound = FALSE;

            PrintCert(pCert);

        } while(pCert != NULL);

		status = ERROR_SUCCESS;
	}
	__finally
	{
		if(pSubjectName != NULL) delete pSubjectName;

        if(pCert != NULL) CertFreeCertificateContext(pCert);

		if(hStore != NULL) CertCloseStore(hStore, 0);
	}    

	return status;
}

void PrintCert(PCCERT_CONTEXT pCert)
{
HCRYPTPROV            pvkProv       = NULL;
HCRYPTKEY             pk            = NULL;
PCRYPT_KEY_PROV_INFO  pInfo         = NULL;
LPWSTR                pSubjectName  = NULL;
BOOL                  releasePk     = TRUE;
DWORD                 keySpec       = 0;
DWORD                 length        = 0;
LPSTR                 pCertName     = NULL;
LPBYTE                pCertHash     = NULL;
int                   status        = ERROR_SUCCESS;

	__try
	{
		if(CertGetCertificateContextProperty(pCert, CERT_KEY_PROV_INFO_PROP_ID, NULL, &length) == FALSE)
		{
			status = GetLastError();

			if(CRYPT_E_NOT_FOUND == status)
			{
				printf("This certificate has no associated private key.\r\n");
			}
			else
			{
				printf("CertGetCertificateContextProperty failed.  Error = 0x%08X", status);
			}

			__leave;
		}

		pInfo = (PCRYPT_KEY_PROV_INFO)new BYTE[length];
    	
		if(CertGetCertificateContextProperty(pCert, CERT_KEY_PROV_INFO_PROP_ID, pInfo, &length) == FALSE)
		{
			status = GetLastError();
			printf("CertGetCertificateContextProperty failed.  Error = 0x%08X", status);
			__leave;
		}

        length = CertGetNameString(pCert, CERT_NAME_FRIENDLY_DISPLAY_TYPE, 0, &pCert->pCertInfo->Subject, NULL, 0);

        pCertName = new char[length];

        CertGetNameString(pCert, CERT_NAME_FRIENDLY_DISPLAY_TYPE, 0, &pCert->pCertInfo->Subject, pCertName, length);

        printf("Cert Name:              %s\r\n", pCertName);

        length = 0;

        printf("Cert Hash:              ");
        if(CertGetCertificateContextProperty(pCert, CERT_HASH_PROP_ID, NULL, &length))
        {
            pCertHash = new BYTE[length];

            if(CertGetCertificateContextProperty(pCert, CERT_HASH_PROP_ID, pCertHash, &length))
            {
                for(DWORD i = 0; i < length; i++)
                {
                    printf("%02X", *(pCertHash + i));
                }
            }
        }
        printf("\r\n");
		
        printf("Key Container Location: %s\r\n", pInfo->dwFlags & CRYPT_MACHINE_KEYSET ? "Machine" : "User");
		printf("Key Container Name:     %S\r\n", pInfo->pwszContainerName);
        printf("Provider Name:          %S\r\n", pInfo->pwszProvName);
        if(pInfo->dwProvType < (sizeof(providers) / sizeof(providers[0])))
        {
            printf("Provider Type:          %s\r\n", providers[pInfo->dwProvType]);
        }
        else
        {
            printf("Provider Type:          %d - <UNKNOWN>\r\n", pInfo->dwProvType);
        }		
		printf("Key Spec:               %s\r\n", (AT_KEYEXCHANGE == pInfo->dwKeySpec) ? "Exchange" : "Signature");

		if(CryptAcquireCertificatePrivateKey(pCert, 0, NULL, &pvkProv, &keySpec, &releasePk) == FALSE)
		{
			status = GetLastError();
			printf("Cannot access the private key container.  CryptAcquireCertificatePrivateKey failed.  Error = 0x%08X", status);
			__leave;
		}

		if(CryptGetUserKey(pvkProv, keySpec, &pk) == FALSE)
		{
			status = GetLastError();
			printf("Cannot access the private key.  CryptGetUserKey failed.  Error = 0x%08X", status);
			__leave;
		}

		printf("\r\nThe private key is accessible\r\n");
	}
	__finally
	{
        if(pCertName != NULL) delete pCertName;

        if(pCertHash != NULL) delete pCertHash;

        if(pInfo != NULL) delete pInfo;

		if(pk != NULL) CryptDestroyKey(pk);

		if((pvkProv != NULL) && releasePk) CryptReleaseContext(pvkProv, 0);
	}
}
